/* Header file for GRETA P3 sort codes
   Author:  Fallon 2005
 */

#define TIME_STEPS 50      /* number of time steps calculated for each segment (p) */
#define TOT_SEGS   36      /* total number of segments in crystal (t2) */

/*--------------------------------------------------------------------*/

typedef struct {
  long long int time;                    /* fpga time stamp */
  int offset;                            /* time offset applied to time stamp*/
  int id;                                /* crystal id */
  float total_energy;                    /* total energy in crystal */ 
  float seg_energy[TOT_SEGS];            /* net energy in each segment */
  float signal[TOT_SEGS][TIME_STEPS];    /* measured segment signals */
  float central[TIME_STEPS];             /* measured central contact signals */
} Event_Signal;


/*--------------------------------------------------------------------*/

double theta[120] = {90,90,90,90,90,90,
		     90,90,90,90,90,90,
		     90,90,90,90,90,90,
		     90,90,90,90,90,90,
		     90,90,90,90,90,90,
		     90,90,90,90,90,90};


/* P3 Nov 2005 LBNL Inbeam */
/*{81.65,82.34,83.08,83.84,
  98.35,97.66,96.92,96.16,
  106.88,105.46,103.93,102.40,
  98.35,97.66,96.92,96.16,		      
  81.65, 83.08,83.84,
  73.12,74.54,76.06,77.60, 155.0};
*/    
/*--------------------------------------------------------------------*/
int lookup[36] = {1,1,1,1,1,1,
		  1,1,1,1,1,1,
		  1,1,1,1,1,1,
		  1,1,1,1,1,1,
		  1,1,1,1,1,1,
		  1,1,1,1,1,1};
/*--------------------------------------------------------------------*/

/* maps segment number 1-36 to layer, phi  P3 MSU 2006:  */
/* a-f == layers, numbers == phi */

char *seg[36] = {"a1","a2","a3","a4","a5","a6",
		 "b1","b2","b3","b4","b5","b6",
		 "c1","c2","c3","c4","c5","c6",
		 "d1","d2","d3","d4","d5","d6",
		 "e1","e2","e3","e4","e5","e6",
		 "f1","f2","f3","f4","f5","f6"};

/* map segment number to ichn (electronics channel number) P3 MSU 2006 */
/* [crystal][ichn]==segment number */

int ichn2seg[1][40] = {{0, 7, 2, 3, 4, 5,
			6, 1, 8, 9, 10,11,
			24,19,14,15,16,-1,
			18,-1,-1,21,22,23,
			12,31,26,27,34,29,
			30,13,32,33,28,35,
			20,17,25,39}};

 /* Maps input segment number to output .dat format */
 /* signum[i] = j ; i = segment number; j = signal position in output file*/

/* P3 2005 */
  int signum[36] = { 0, 6,12,18,24,30,      //  layer 1 segments a-f
		     1, 7,13,19,25,31,      //  layer 2 
		     2, 8,14,20,26,32,     //   layer 3
		     3, 9,15,21,27,33,      //  layer 4 
		     4,10,16,22,28,34,     //   layer 5 
		     5,11,17,23,29,35};    //   layer 6 
   
/* for P2 */
/*
  int signum[36] = { 0, 1, 2, 3, 4, 5,      //  segment a
		   6, 7, 8, 9, 10,11,       //  seg b 
		   12,13,14,15,16,17,     //   seg c
		   18,19,20,21,22,23,      //  seg d
		   24,25,26,27,28,29,     //   seg e
		   30,31,32,33,34,35};    //   seg f

*/


/*--------------------------------------------------------------------*/

struct spePrefix {
  int reclA;
  unsigned int titleA;
  unsigned int titleB;
  int dim;
  int a1;
  int a2;
  int a3;
  int reclB;
} spePre = {24, 0, 0, 0, 1, 1, 1, 24};
void writeSpectra(const char *filename, int *buf, int dim) {
  
  FILE *fp_spectra;
  int recl;
  float *fbuf;
  int i;
  
  fp_spectra = fopen(filename, "w");
  if (fp_spectra == NULL) {
    fprintf(stderr, "unable to open file for spectra");
    exit(1);
  } 
  
  spePre.dim = dim;
  recl = sizeof(float) * dim;
  
  fbuf = (float *) calloc(dim, sizeof(float));
  if (fbuf == NULL) {
    fprintf(stderr, "calloc error\n");
    exit(2);
  }
  for (i = 0; i < dim; i++) {
    fbuf[i] = (float) buf[i];
  }
  
  (void) fwrite(&spePre, sizeof(struct spePrefix), 1, fp_spectra);
  (void) fwrite(&recl, sizeof(int), 1, fp_spectra);
  (void) fwrite(fbuf, sizeof(float), dim, fp_spectra); 
  (void) fwrite(&recl, sizeof(int), 1, fp_spectra);
  
  fclose(fp_spectra);
  free(fbuf);
}
/*--------------------------------------------------------------------*/

/* function returns a number between -0.5 and 0.5 */
/* cycles in 0.1 steps */
float randNum() {
  static float r; /* cast as static so function remembers previous value of r */
  //r = r+0.1;
  //if (r > 0.5) r = -0.5;
  r = 0.01*(rand()%100) -0.5;
  //printf("%f \n",r);
  return r;
}

